﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DataModel
{
  [AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = true)]
  public class ExpectedValidationExceptionAttribute: ExpectedExceptionBaseAttribute
  {
    public string MemberName { get; set; }

    protected override void Verify(Exception exception)
    {
      this.RethrowIfAssertException(exception);

      IEnumerable<string> actualMemberNames = null;

      if (string.IsNullOrEmpty(this.MemberName))
      {
        if (exception is ValidationException) 
          return;

        var aggregateException = exception as AggregateException;
        if (aggregateException != null && aggregateException.InnerExceptions.OfType<ValidationException>().Any())
          return;
      }
      else 
      {
        var validationException = exception as ValidationException;
        if (validationException != null)
          actualMemberNames = validationException.ValidationResult.MemberNames;

        var aggregateException = exception as AggregateException;
        if (aggregateException != null)
          actualMemberNames = aggregateException.InnerExceptions.OfType<ValidationException>().SelectMany(e => e.ValidationResult.MemberNames ?? new string[0]);

        if (actualMemberNames != null && actualMemberNames.Contains(this.MemberName))
          return;
      }

      string message = "ValidationException or AggregateException was expected";
      if (!string.IsNullOrEmpty(this.MemberName))
        message += " with member name " + this.MemberName;
      message += ". Instead " + exception.GetType().FullName + " was thrown";
      if (actualMemberNames != null && actualMemberNames.Any())
        message += " with member name(s) " + string.Join(", ", actualMemberNames);
      message += ".";
      throw new AssertFailedException(message);
    }
  }
}
