﻿using System;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DataModel
{
  [TestClass]
  public class ProductTest
  {
    [TestMethod]
    [ExpectedValidationException(MemberName = "ProductName")]
    public void ProductName_IsRequired()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var product = new Product();
        product.ProductName = string.Empty;
        context.Products.AddObject(product);
        context.SaveChanges();
      }
    }

    [TestMethod]
    [ExpectedValidationException(MemberName = "ProductName")]
    public void ProductName_MustBe40CharactersOrLess()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var product = new Product();
        product.ProductName = new String('T', 41);
        context.Products.AddObject(product);
        context.SaveChanges();
      }
    }

    [TestMethod]
    [ExpectedValidationException(MemberName = "ProductName")]
    public void ProductName_MustBeUnique()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var p1 = new Product() { ProductName = "Test", UnitPrice = 1 };
        var p2 = new Product() { ProductName = "Test", UnitPrice = 1 };
        context.Products.AddObject(p1);
        context.Products.AddObject(p2);
        context.SaveChanges();
      }
    }

    [TestMethod]
    public void UnitsInStock_IsZeroByDefault()
    {
      var product = new Product();
      Assert.AreEqual((short)0, product.UnitsInStock);
    }

    [TestMethod]
    [ExpectedValidationException(MemberName = "UnitPrice")]
    public void UnitPrice_IsRequired()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var product = new Product();
        product.UnitPrice = null; 
        context.Products.AddObject(product);
        context.SaveChanges();
      }
    }

    // Test that does not rely on ExpectedValidationException
    //[TestMethod]
    //public void UnitPrice_IsRequired()
    //{
    //  using (new TransactionScope())
    //  using (var context = new NorthwindEntities())
    //  {
    //    var product = new Product();
    //    product.UnitPrice = null;
    //    context.Products.AddObject(product);
    //    try
    //    {
    //      context.SaveChanges();
    //      Assert.Fail();
    //    }
    //    catch (ValidationException e)
    //    {
    //      Assert.IsNotNull(e.ValidationResult.MemberNames);
    //      Assert.IsTrue(e.ValidationResult.MemberNames.Contains("UnitPrice"));
    //    }
    //    catch
    //    {
    //      Assert.Fail();
    //    }
    //  }
    //}


    [TestMethod]
    [ExpectedValidationException(MemberName = "UnitPrice")]
    public void UnitPrice_MustBeGreaterThanZero()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var product = new Product();
        product.UnitPrice = -1;
        context.Products.AddObject(product);
        context.SaveChanges();
      }
    }
  }
}
