﻿using System.Security.Principal;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Unleashed.DataAnnotations.Tests
{
  [TestClass]
  public class AuthorizationRuleTest
  {
    [TestMethod]
    public void IsMatchReturnsTrueWhenUserHasMatchingRole()
    {
      var administrator = new GenericPrincipal(new GenericIdentity("Adam"), roles: new[] { "Administrator" });
      var user = new GenericPrincipal(new GenericIdentity("Udi"), roles: new[] { "User" });
      var rule = new AuthorizationRule(true, Actions.All, roles: new[] { "Administrator" }, users: new string[0]);
      Assert.IsTrue(rule.IsMatch(administrator));
      Assert.IsFalse(rule.IsMatch(user));
    }

    [TestMethod]
    public void IsMatchReturnsTrueWhenUserHasMatchingName()
    {
      var adam = new GenericPrincipal(new GenericIdentity("Adam"), null);
      var udi = new GenericPrincipal(new GenericIdentity("Udi"), null);
      var rule = new AuthorizationRule(true, Actions.All, roles: new string[0], users: new[] { "Adam" });
      Assert.IsTrue(rule.IsMatch(adam));
      Assert.IsFalse(rule.IsMatch(udi));
    }

    [TestMethod]
    public void IsMatchReturnsTrueForAllUsersWildcard()
    {
      var user = new GenericPrincipal(new GenericIdentity("Udi"), null);
      var rule = new AuthorizationRule(true, Actions.All, roles: new string[0], users: new[] { "*" });
      Assert.IsTrue(rule.IsMatch(user));
    }

    [TestMethod]
    public void IsMatchReturnsTrueForUnauthenticatedUsersWildcard()
    {
      var user = new GenericPrincipal(new GenericIdentity(string.Empty), null);
      var rule = new AuthorizationRule(true, Actions.All, roles: new string[0], users: new[] { "?" });
      Assert.IsTrue(rule.IsMatch(user));
    }

    [TestMethod]
    public void ToStringIncludesAllowOrDeny()
    {
      var allowRule = new AuthorizationRule(true, Actions.All, new string[0], new string[0]);
      StringAssert.Contains(allowRule.ToString(), "allow");

      var denyRule = new AuthorizationRule(false, Actions.All, new string[0], new string[0]);
      StringAssert.Contains(denyRule.ToString(), "deny");
    }

    [TestMethod]
    public void ToStringIncludesActions()
    {
      var rule = new AuthorizationRule(true, Actions.All, new string[0], new string[0]);
      StringAssert.Contains(rule.ToString(), "All");

      var partialRule = new AuthorizationRule(true, Actions.Read | Actions.Insert, new string[0], new string[0]);
      StringAssert.Contains(partialRule.ToString(), "Read");
      StringAssert.Contains(partialRule.ToString(), "Insert");
    }

    [TestMethod]
    public void ToStringIncludesRoles()
    {
      var rule = new AuthorizationRule(true, Actions.All, new[] {"Role1Name", "Role2Name" }, new string[0]);
      StringAssert.Contains(rule.ToString(), "Role1Name");
      StringAssert.Contains(rule.ToString(), "Role2Name");
    }

    [TestMethod]
    public void ToStringIncludesUsers()
    {
      var rule = new AuthorizationRule(true, Actions.All, new string[0], new[] { "User1Name", "User2Name" });
      StringAssert.Contains(rule.ToString(), "User1Name");
      StringAssert.Contains(rule.ToString(), "User2Name");
    }


  }
}
