﻿namespace Unleashed.DataAnnotations
{
  using System;
  using System.Collections.Generic;
  using System.Collections.ObjectModel;
  using System.Linq;
  using System.Security.Principal;
  using System.Text;

  public class AuthorizationRule
  {
    internal AuthorizationRule(bool authorize, Actions actions, IList<string> roles, IList<string> users)
    {
      this.Authorize = authorize;
      this.Actions = actions;
      this.Roles = new ReadOnlyCollection<string>(roles);
      this.Users = new ReadOnlyCollection<string>(users);
    }

    public bool Authorize { get; private set; }
    public Actions Actions { get; private set; }
    public ReadOnlyCollection<string> Roles { get; private set; }
    public ReadOnlyCollection<string> Users { get; private set; }

    public override string ToString()
    {
      var result = new StringBuilder();
      result.Append("<");
      result.Append(this.Authorize ? "allow" : "deny");
      result.AppendFormat(" actions='{0}'", this.Actions);
      if (this.Roles.Count > 0)
        result.AppendFormat(" roles='{0}'", string.Join(", ", this.Roles.ToArray()));
      if (this.Users.Count > 0)
        result.AppendFormat(" users='{0}'", string.Join(", ", this.Users.ToArray()));
      result.Append("/>");
      return result.ToString();
    }

    internal bool IsMatch(IPrincipal principal)
    {
      foreach (string role in this.Roles)
      {
        if (principal.IsInRole(role))
          return true;
      }

      foreach (string user in this.Users)
      {
        if (user == "*")
          return true;
        if (user == "?" && !principal.Identity.IsAuthenticated)
          return true;
        if (string.Compare(user, principal.Identity.Name, StringComparison.InvariantCultureIgnoreCase) == 0)
          return true;
      }

      return false;
    }
  }
}
