namespace Unleashed.DataAnnotations
{
  using System;
  using System.Linq;
  using System.Reflection;

  [AttributeUsage(AttributeTargets.Class)]
  public class CustomQueryAttribute : Attribute
  {
    private MethodInfo methodInfo;

    public CustomQueryAttribute(Type type, string method)
    {
      this.Type = type;
      this.Method = method;
    }

    public string Method { get; private set; }

    public Type Type { get; private set; }

    public IQueryable GetQueryable(IQueryable source)
    {
      if (this.methodInfo == null)
      {
        if (this.Type == null)
          throw new InvalidOperationException("The CustomQueryAttribute.Type was not specified.");

        if (string.IsNullOrEmpty(this.Method))
          throw new InvalidOperationException("The CustomQueryAttribute.Method was not specified.");

        MethodInfo methodInfo = this.Type.GetMethod(this.Method, BindingFlags.Public | BindingFlags.Static);

        if (methodInfo == null)
          throw new InvalidOperationException(string.Format("The CustomQueryAttribute method '{0}' does not exist in type '{1}' or is not public and static.", this.Method, this.Type.Name));

        if (!typeof(IQueryable).IsAssignableFrom(methodInfo.ReturnType))
          throw new InvalidOperationException(string.Format("The CustomQueryAttribute method '{0}' in type '{1}' must return System.Linq.IQueryable.", this.Method, this.Type.Name));

        ParameterInfo[] parameters = methodInfo.GetParameters();
        if (parameters.Length != 1 || parameters.Length > 0 && parameters[0].ParameterType != typeof(IQueryable))
          throw new InvalidOperationException(string.Format("The CustomQueryAttribute method '{0}' in type '{1}' must match the expected signature: public static IQueryable {0}(IQueryable source).", this.Method, this.Type.Name));

        this.methodInfo = methodInfo;
      }

      return (IQueryable)this.methodInfo.Invoke(null, new object[] {source});
    }
  }
}