namespace Unleashed.DynamicData
{
  using System;
  using System.Collections;
  using System.Collections.Concurrent;
  using System.Collections.Specialized;
  using System.ComponentModel.DataAnnotations;
  using System.Linq;
  using System.Reflection;
  using System.Web.DynamicData;
  using System.Web.UI.WebControls;

  public class UnleashedEnumerationFieldTemplate : UnleashedFieldTemplate
  {
    private static readonly ConcurrentDictionary<Type, IOrderedDictionary> enumCache = new ConcurrentDictionary<Type, IOrderedDictionary>();

    public override string FieldValueString
    {
      get
      {
        object fieldValue = this.FieldValue;
        Type enumType = this.Column.GetEnumType();
        if (fieldValue != null && enumType != null)
        {
          fieldValue = Enum.ToObject(enumType, fieldValue); // Convert numeric value to enum, if necessary
          IOrderedDictionary enumDictionary = GetEnumDictionary(enumType);
          var enumName = (string)enumDictionary[fieldValue];
          if (!string.IsNullOrEmpty(enumName))
            return enumName;
        }

        return base.FieldValueString;
      }
    }

    protected new void PopulateListControl(ListControl listControl)
    {
      Type enumType = this.Column.GetEnumType();
      Type underlyingType = Enum.GetUnderlyingType(enumType);
      IOrderedDictionary enumDictionary = GetEnumDictionary(enumType);
      foreach (DictionaryEntry entry in enumDictionary)
      {
        var enumName = (string)entry.Value;
        object enumValue = Convert.ChangeType(entry.Key, underlyingType);
        listControl.Items.Add(new ListItem(enumName, enumValue.ToString()));
      }
    }

    private static IOrderedDictionary GetEnumDictionary(Type enumType)
    {
      IOrderedDictionary enumDictionary;
      if (!enumCache.TryGetValue(enumType, out enumDictionary))
      {
        enumDictionary = BuildEnumDictionary(enumType);
        enumCache.TryAdd(enumType, enumDictionary);
      }

      return enumDictionary;
    }

    private static IOrderedDictionary BuildEnumDictionary(Type enumType)
    {
      var dictionary = new OrderedDictionary();
      FieldInfo[] enumFields = enumType.GetFields(BindingFlags.Static | BindingFlags.GetField | BindingFlags.Public);
      foreach (FieldInfo enumField in enumFields)
      {
        object enumValue = enumField.GetValue(null);
        DisplayAttribute annotation = enumField.GetCustomAttributes(typeof(DisplayAttribute), false).OfType<DisplayAttribute>().FirstOrDefault();
        string enumName = annotation != null ? annotation.GetName() : enumValue.ToString();
        dictionary.Add(enumValue, enumName);
      }

      return dictionary;
    }
  }
}