namespace Unleashed.DynamicData
{
  using System;
  using System.ComponentModel;
  using System.Web.DynamicData;
  using System.Web.UI.WebControls;

  public class UnleashedLabel : Label
  {
    [DefaultValue("")]
    public string DataField
    {
      get
      {
        object obj = this.ViewState["DataField"];
        return obj != null ? (string)obj : string.Empty;
      }
      set
      {
        this.ViewState["DataField"] = value;
      }
    }

    protected override void OnInit(EventArgs e)
    {
      base.OnInit(e);
      MetaTable table = this.FindMetaTable();
      if (table == null)
        throw new InvalidOperationException("The UnleashedLabel needs to exist inside a data control that is bound to a data source that supports Dynamic Data.");
      MetaColumn column = table.GetColumn(this.DataField);
      this.Text = column.Prompt ?? column.DisplayName;
    }

    protected override void OnPreRender(EventArgs e)
    {
      base.OnPreRender(e);
      var fieldTemplate = this.FindFieldTemplate(this.DataField) as FieldTemplateUserControl;
      if (fieldTemplate != null && fieldTemplate.DataControl != null)
        this.AssociatedControlID = fieldTemplate.DataControl.GetUniqueIDRelativeTo(this);
    }
  }
}