﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web.DynamicData;

namespace Unleashed.DynamicData
{
  public class UnleashedValidator : DynamicValidator
  {
    public bool HandleException(Exception exception)
    {
      bool exceptionHandled = false;
      foreach (UnleashedValidator validator in this.Page.Validators.OfType<UnleashedValidator>())
      {
        validator.ValidateException(exception);
        exceptionHandled |= validator.ValidationException != null;
      }

      return exceptionHandled;
    }

    protected override void ValidateException(Exception exception)
    {
      base.ValidateException(exception);
      if (this.ValidationException == null)
      {
        var aggregateException = exception as AggregateException;
        if (aggregateException != null)
          this.HandleAggregateException(aggregateException);
      }
    }

    private IEnumerable<string> GetColumnNames()
    {
      if (this.Column != null)
      {
        yield return this.Column.Name;
        var foreignKey = this.Column as MetaForeignKeyColumn;
        if (foreignKey != null)
        {
          foreach (string columnName in foreignKey.ForeignKeyNames)
            yield return columnName;
        }

        yield break;
      }

      yield return string.Empty;
    }

    private void HandleAggregateException(AggregateException aggregate)
    {
      IDictionary<string, ValidationException> dictionary = GetValidationExceptions(aggregate);
      foreach (string columnName in this.GetColumnNames())
      {
        ValidationException validationException;
        if (dictionary.TryGetValue(columnName, out validationException))
        {
          this.ValidationException = validationException;
          return;
        }
      }
    }

    private static IDictionary<string, ValidationException> ExtractValidationExceptions(AggregateException aggregate)
    {
      var dictionary = new Dictionary<string, ValidationException>();
      foreach (ValidationException exception in aggregate.InnerExceptions.OfType<ValidationException>())
      {
        foreach (string memberName in GetMemberNames(exception.ValidationResult))
        {
          if (!dictionary.ContainsKey(memberName))
            dictionary.Add(memberName, exception);
        }
      }

      return dictionary;
    }

    private static IEnumerable<string> GetMemberNames(ValidationResult validationResult)
    {
      foreach (string memberName in validationResult.MemberNames)
        yield return memberName ?? string.Empty;

      if (!validationResult.MemberNames.Any())
        yield return string.Empty;
    }

    private static IDictionary<string, ValidationException> GetValidationExceptions(AggregateException aggregate)
    {
      var dictionary = (IDictionary<string, ValidationException>)HttpContext.Current.Items[aggregate];
      if (dictionary == null)
      {
        dictionary = ExtractValidationExceptions(aggregate);
        HttpContext.Current.Items[aggregate] = dictionary;
      }

      return dictionary;
    }
  }
}