﻿using System;
using System.Collections;
using System.Linq;
using System.Web.DynamicData;
using System.Web.UI;
using System.Web.UI.WebControls;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.Filters
{
  public partial class ForeignKeyFilter : UnleashedFilterTemplate
  {
    private const string NullValueString = "[null]";

    private new MetaForeignKeyColumn Column
    {
      get { return (MetaForeignKeyColumn)base.Column; }
    }

    public override Control FilterControl
    {
      get { return this.dropDownList; }
    }

    protected void Page_Init(object sender, EventArgs e)
    {
      if (!this.Page.IsPostBack)
      {
        if (!this.Column.IsRequired)
        {
          this.dropDownList.Items.Add(new ListItem("[Not Set]", NullValueString));
        }

        this.PopulateListControl(this.dropDownList);

        // Set the initial value if there is one
        string initialValue = this.DefaultValue;
        if (!String.IsNullOrEmpty(initialValue))
        {
          this.dropDownList.SelectedValue = initialValue;
        }
      }
    }

    protected void DropDownList_SelectedIndexChanged(object sender, EventArgs e)
    {
      this.OnFilterChanged();
    }

    public override IQueryable GetQueryable(IQueryable source)
    {
      string selectedValue = this.dropDownList.SelectedValue;
      if (string.IsNullOrEmpty(selectedValue))
      {
        return source;
      }

      if (selectedValue == NullValueString)
      {
        return ApplyEqualityFilter(source, this.Column.Name, null);
      }

      IDictionary dict = new Hashtable();
      this.Column.ExtractForeignKey(dict, selectedValue);
      foreach (DictionaryEntry entry in dict)
      {
        var key = (string)entry.Key;
        if (this.DefaultValues != null)
        {
          this.DefaultValues[key] = entry.Value;
        }

        source = ApplyEqualityFilter(source, this.Column.GetFilterExpression(key), entry.Value);
      }

      return source;
    }
  }
}