﻿namespace WebApplication.DynamicData.Filters
{
  using System;
  using System.Linq;
  using System.Linq.Expressions;
  using System.Web.DynamicData;
  using System.Web.UI;
  using System.Web.UI.WebControls;
  using Unleashed.DynamicData;

  public partial class ParentFilter : UnleashedFilterTemplate
  {
    private object dataSourceContext;
    private MetaForeignKeyColumn foreignKeyColumn;

    public string DisplayField { get; set; }

    public string ForeignKeyField { get; set; }

    public override Control FilterControl
    {
      get { return this.filter.FilterTemplate.FilterControl; }
    }

    public new MetaForeignKeyColumn ForeignKeyColumn
    {
      get
      {
        if (this.foreignKeyColumn == null)
        {
          if (!string.IsNullOrEmpty(this.ForeignKeyField))
            this.foreignKeyColumn = (MetaForeignKeyColumn)this.Column.Table.GetColumn(this.ForeignKeyField);
          else
            this.foreignKeyColumn = base.ForeignKeyColumn;
        }

        return this.foreignKeyColumn;
      }
    }

    protected void Filter_Init(object sender, EventArgs args)
    {
      this.filter.DataSource = this.DataSource;
      this.filter.Table = this.ForeignKeyColumn.ParentTable;
      this.filter.DataField = this.DisplayField;
      ((EntityDataSource)this.DataSource).ContextCreated += this.DataSourceContextCreated;
      ((EntityDataSource)this.DataSource).Selecting += this.DataSourceSelecting;
    }

    public override IQueryable GetQueryable(IQueryable childQuery)
    {
      MetaTable parentTable = this.ForeignKeyColumn.ParentTable;
      IQueryable emptyQuery = parentTable.GetQuery(this.dataSourceContext);
      IQueryable parentQuery = this.filter.FilterTemplate.GetQueryable(emptyQuery);
      if (parentQuery != emptyQuery)
      {
        MetaTable childTable = this.ForeignKeyColumn.Table;
        MetaColumn childColumn = childTable.GetColumn(this.ForeignKeyColumn.ForeignKeyNames[0]);
        MetaColumn parentColumn = parentTable.PrimaryKeyColumns[0];

        MethodCallExpression joinCall = Expression.Call(
          typeof(Queryable),
          "Join",
          new[]
            {
              childQuery.ElementType,
              parentQuery.ElementType,
              childColumn.EntityTypeProperty.PropertyType,
              childQuery.ElementType
            },
          new[]
            {
              childQuery.Expression,
              parentQuery.Expression,
              BuildChildKeySelector(childQuery.ElementType, childColumn.Name),
              BuildParentKeySelector(parentQuery.ElementType, parentColumn.Name, childColumn.EntityTypeProperty.PropertyType),
              BuildResultSelector(childQuery.ElementType, parentQuery.ElementType)
            });

        childQuery = childQuery.Provider.CreateQuery(joinCall);
      }

      return childQuery;
    }

    private static LambdaExpression BuildChildKeySelector(Type entityType, string propertyName)
    {
      ParameterExpression parameter = Expression.Parameter(entityType, "child");
      MemberExpression property = Expression.Property(parameter, propertyName);
      return Expression.Lambda(property, parameter);
    }

    private static LambdaExpression BuildParentKeySelector(Type entityType, string propertyName, Type expectedPropertyType)
    {
      ParameterExpression parameter = Expression.Parameter(entityType, "parent");
      MemberExpression property = Expression.Property(parameter, propertyName);
      if (property.Type == expectedPropertyType)
        return Expression.Lambda(property, parameter);

      UnaryExpression convertedProperty = Expression.Convert(property, expectedPropertyType);
      return Expression.Lambda(convertedProperty, parameter);
    }

    private static LambdaExpression BuildResultSelector(Type childEntityType, Type parentEntityType)
    {
      ParameterExpression childParameter = Expression.Parameter(childEntityType, "child");
      ParameterExpression parentParameter = Expression.Parameter(parentEntityType, "parent");
      return Expression.Lambda(childParameter, childParameter, parentParameter);
    }

    private void DataSourceContextCreated(object sender, EntityDataSourceContextCreatedEventArgs e)
    {
      this.dataSourceContext = e.Context;
    }

    private void DataSourceSelecting(object sender, EntityDataSourceSelectingEventArgs e)
    {
      if (string.IsNullOrEmpty(e.SelectArguments.SortExpression))
        e.SelectArguments.SortExpression = string.Join(",", this.ForeignKeyColumn.Table.PrimaryKeyColumns.Select(c => c.Name));
    }
  }
}